import moment from 'moment';
import _ from 'lodash';

/**
 * 轴宽度, 用于计算多轴显示时, 轴线偏移和绘图区域尺寸
 */
const axisWidth = 40;

/**
 * 图表系列名称格式化
 * @param {*} data 
 * @param {boolean} contrast 是否为同期对比
 * @param {*} contrastOption 同期对比周期配置, day|month
 * @returns 
 */
const nameFormatter = (data, contrast, contrastOption) => {
  const { equipmentName, sensorName, unit, dataModel, dateFrom, dateTo } = data;
  let name = `${equipmentName}-${sensorName}`;
  if (contrast) {
    const time = dateFrom.slice(0, contrastOption === 'day' ? 10 : 7).replace(/-/g, '');
    name = `${name}-${time}`;
  }
  return name;
};

/**
 * 图表系列数据格式化
 * @param {*} data 
 * @param {boolean} contrast 是否为同期对比
 * @param {*} contrastOption 同期对比周期配置, day|month
 * @returns 图表系列数据, [[DateTime, value]]
 */
const dataAccessor = (data, contrast, contrastOption) => {
  const { dataModel } = data;
  const formatStr = contrastOption === 'day' ? '2020-01-01 HH:mm:00' : '2020-01-DD HH:mm:00';
  return dataModel.map(item => {
    const time = contrast ? moment(item.pt).format(formatStr) : item.pt;
    return [moment(time).valueOf(), item.pv];
  });
};

/**
 * 面积图配置(目前默认曲线图)
 * @param {*} data 数据项
 * @returns null/areaStyle, 为null显示曲线图, 为areaStyle对象显示为面积图.
 */
const areaStyleFormatter = (data) => {
  const { sensorName } = data;
  return sensorName && sensorName.indexOf('流量') > -1 ? {} : null;
};


/**
 * 图表配置项生成
 * @param {*} dataSource 数据源
 * @param {*} cusOption 自定义属性
 * @param {*} contrast 是否为同期对比
 * @param {*} contrastOption 同期对比周期配置, day|month
 * @param {*} smooth ture/false, 曲线/折线
 * @param {*} config 额外配置信息
 */
const optionGenerator = (dataSource, cusOption, contrast, contrastOption, smooth, config) => {
  const needUnit = _.get(config, 'needUnit', false);
  // 自定义属性
  const restOption = _.pick(cusOption, ['title',]);

  // 一种指标一个y轴
  const yAxisMap = new Map();
  dataSource.forEach((item, index) => {
    const { sensorName, unit } = item;
    const key = sensorName;

    if (!yAxisMap.has(key)) {
      const i = yAxisMap.size;
      const axis = {
        type: 'value',
        name: needUnit ? unit : null,
        position: i % 2 === 0 ? 'left' : 'right',
        offset: Math.floor(i / 2) * axisWidth,
        axisLabel: {
          formatter: (value) => value > 100000 ? `${value / 1000}k` : value,
        },
        axisLine: {
          show: true,
        },
        nameTextStyle: {
          align: i % 2 === 0 ? 'right' : 'left'
        },
      }
      yAxisMap.set(key, axis);
    }
  });
  const yAxis = yAxisMap.size > 0 ? [...yAxisMap.values()] : { type: 'value' };

  // 根据y轴个数调整边距
  const leftNum = Math.ceil(yAxisMap.size / 2);
  const rightNum = Math.floor(yAxisMap.size / 2);
  const grid = {
    top: needUnit ? 80 : 60,
    left: 10 + leftNum * axisWidth,
    right: rightNum === 0 ? 20 : rightNum * axisWidth,
  };

  // 根据"指标名称"分类yAxis
  const yAxisInterator = (() => {
    const map = new Map();
    let current = -1;
    const get = (name) => map.has(name) ? map.get(name) : map.set(name, ++current).get(name);
    return { get }
  })();
  

  const series = dataSource.map(item => {
    const { sensorName, unit } = item;
    const name = nameFormatter(item, contrast, contrastOption);
    const data = dataAccessor(item, contrast, contrastOption);
    const type = 'line';
    const areaStyle = areaStyleFormatter(item);
    const yAxisIndex = yAxisInterator.get(sensorName);
    return {
      notMerge: true,
      name,
      type,
      data,
      areaStyle,
      yAxisIndex,
      smooth,
      unit,
    }
  });

  // 由于series更新后，没有的数据曲线仍然停留在图表区上，导致图表可视区范围有问题
  const min = Math.min(...series.map(item => item.data?.[0]?.[0]).filter(item => item !== undefined));
  const max = Math.max(...series.map(item => item.data?.[item.data.length - 1]?.[0]).filter(item => item !== undefined));
  const xAxis = { type: 'time', min, max };

  const tooltipTimeFormat = !contrast ? 'YYYY-MM-DD HH:mm:ss' : contrastOption === 'day' ? 'HH:mm' : 'DD HH:mm';
  const tooltip = { timeFormat: tooltipTimeFormat };

  
  return {
    yAxis,
    grid,
    xAxis,
    series,
    tooltip,
    ...restOption,
  }
};

export default optionGenerator;
